package com.qhtx.eta.domain.task;

import com.alibaba.cloud.nacos.NacosConfigManager;
import com.alibaba.cloud.nacos.NacosConfigProperties;
import com.alibaba.nacos.api.config.ConfigChangeEvent;
import com.alibaba.nacos.api.config.ConfigChangeItem;
import com.alibaba.nacos.api.exception.NacosException;
import com.alibaba.nacos.client.config.listener.impl.AbstractConfigChangeListener;
import com.qhtx.eta.domain.config.ScheduleConfig;
import com.qhtx.eta.domain.config.ScheduleTaskConfig;
import com.qhtx.eta.domain.job.ETADataSyncJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


@Component
@Slf4j
public class TaskConfigChangeListener extends AbstractConfigChangeListener {

    @Resource
    private NacosConfigManager nacosConfigManager;

    @Resource
    private NacosConfigProperties nacosConfigProperties;

    @Resource
    private ETADataSyncJob etadataSyncJob;

    @Resource
    private FixedDelayTaskRegistrar fixedDelayTaskRegistrar;


    @Resource
    private ScheduleTaskConfig scheduleTaskConfig;

    @PostConstruct
    public void init() throws NacosException {
        String dataId = String.format("%s-%s.%s", nacosConfigProperties.getPrefix(), nacosConfigProperties.getEnvironment().getActiveProfiles()[0], nacosConfigProperties.getFileExtension());
        nacosConfigManager.getConfigService().addListener(dataId, "DATABASE_GROUP", this);
    }

    @Override
    public void receiveConfigChange(ConfigChangeEvent configChangeEvent) {
        Map<String, List<ConfigChangeItem>> taskMap = configChangeEvent.getChangeItems().stream().filter(changeItem -> changeItem.getKey().contains("eta.scheduled")).collect(Collectors.groupingBy(changeItem -> {
            if (changeItem.getKey().startsWith("eta.scheduled.info")) return "info";
            else if (changeItem.getKey().startsWith("eta.scheduled.data")) return "data";
            return "other";
        }));
        List<ConfigChangeItem> infoTaskList = taskMap.get("info");
        List<ConfigChangeItem> dataTaskList = taskMap.get("data");
        if (!CollectionUtils.isEmpty(infoTaskList)) {
            ScheduleConfig scheduleConfig = scheduleTaskConfig.getInfo();
            infoTaskList.forEach(changeItem -> {
                if (changeItem.getKey().endsWith("interval")) {
                    scheduleConfig.setInterval(Integer.valueOf(changeItem.getNewValue()));
                }
                if (changeItem.getKey().endsWith("initialDelay")) {
                    scheduleConfig.setInitialDelay(Integer.valueOf(changeItem.getNewValue()));
                }
                if (changeItem.getKey().endsWith("enabled")) {
                    scheduleConfig.setEnabled(Boolean.valueOf(changeItem.getNewValue()));
                }
            });
            if (scheduleConfig.getEnabled()) {
                fixedDelayTaskRegistrar.removeFixedDelayTask(new SchedulingRunnable(etadataSyncJob.getClass().getName(), "syncETAInfo", etadataSyncJob));
                fixedDelayTaskRegistrar.addFixedDelayTask(new SchedulingRunnable(etadataSyncJob.getClass().getName(), "syncETAInfo", etadataSyncJob), scheduleConfig.getInterval(), scheduleConfig.getInitialDelay());
            } else {
                fixedDelayTaskRegistrar.removeFixedDelayTask(new SchedulingRunnable(etadataSyncJob.getClass().getName(), "syncETAInfo", etadataSyncJob));
            }
        }
        if (!CollectionUtils.isEmpty(dataTaskList)) {
            ScheduleConfig scheduleConfig = scheduleTaskConfig.getData();
            dataTaskList.forEach(changeItem -> {
                if (changeItem.getKey().endsWith("interval")) {
                    scheduleConfig.setInterval(Integer.valueOf(changeItem.getNewValue()));
                }
                if (changeItem.getKey().endsWith("initialDelay")) {
                    scheduleConfig.setInitialDelay(Integer.valueOf(changeItem.getNewValue()));
                }
                if (changeItem.getKey().endsWith("enabled")) {
                    scheduleConfig.setEnabled(Boolean.valueOf(changeItem.getNewValue()));
                }
            });
            if (scheduleConfig.getEnabled()) {
                fixedDelayTaskRegistrar.removeFixedDelayTask(new SchedulingRunnable(etadataSyncJob.getClass().getName(), "syncETAData", etadataSyncJob));
                fixedDelayTaskRegistrar.addFixedDelayTask(new SchedulingRunnable(etadataSyncJob.getClass().getName(), "syncETAData", etadataSyncJob), scheduleConfig.getInterval(), scheduleConfig.getInitialDelay());
            } else {
                fixedDelayTaskRegistrar.removeFixedDelayTask(new SchedulingRunnable(etadataSyncJob.getClass().getName(), "syncETAData", etadataSyncJob));
            }
        }

    }
}

